package com.tillekesoft.dragtome;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.collections.List;
import javafx.event.EventHandler;
import javafx.scene.input.DragEvent;
import javafx.scene.input.Dragboard;
import javafx.scene.input.TransferMode;

import java.io.File;
import java.util.Collection;

@BA.Version(0.85F)
@BA.Author("Moster67 aka Mikael Osterhed")

@BA.Events(values = { "ReceivedFilePath (Filepath as String)","ReceivedFiles (filelist As List)" })
@BA.ShortName("B4JDragToMe")
public class B4JDragToMe {

	private BA ba;
	private String eventName;
	private String ReceivedFilePath = null;
	private javafx.scene.control.Control control;

	private void ForwardReceivedFilePath(String path) {
		if (ba.subExists(eventName + "_receivedfilepath")) {
			ba.raiseEvent(B4JDragToMe.this, eventName + "_receivedfilepath", new Object[] { path });
		}
	}

	/**
	 * Initialize by setting your ControlView and giving an event-name. For files,
	 * the event ReceivedFilePath returns the filepath/location of the dragged
	 * object while it will return the URL for an http-link.
	 */
	public void Initialize(final BA ba, javafx.scene.control.Control Control, String EventName) {
		this.ba = ba;
		this.eventName = EventName.toLowerCase(BA.cul);
		this.control = Control;
		addControlEvents();
	}

	private void addControlEvents() {
		control.setOnDragOver(new EventHandler<DragEvent>() {

			@Override
			public void handle(DragEvent event) {
				//ba.Log("OnDragOver()");
				if (event.getGestureSource() != control && event.getDragboard().hasFiles()) {
					event.acceptTransferModes(TransferMode.LINK);
				}
				if (event.getGestureSource() != control && event.getDragboard().hasUrl()) {
					event.acceptTransferModes(TransferMode.LINK);
				}

				//event.consume();
			}
		});
		
		control.setOnDragDone(new EventHandler<DragEvent>(){

			@Override
			public void handle(DragEvent event) {
				ba.Log("OnDragDone()");
				ba.Log(event.getEventType().toString());
				Dragboard dragboard = event.getDragboard();
				boolean success = false;
				File newFile = null;

				if (dragboard.hasUrl() && !dragboard.hasFiles()) {
					ReceivedFilePath = dragboard.getUrl();
					success = true;
					ForwardReceivedFilePath(ReceivedFilePath);
					event.setDropCompleted(success);
					event.consume();
				} else if (dragboard.hasFiles()) {
					List fls = new List();
					fls.Initialize();
					java.util.List<File> newFiles = dragboard.getFiles();
					ba.Log("NewFiles.Size = "+newFiles.size());
					for (File s : newFiles) {
						fls.Add(s.getAbsolutePath());
					}
					if (!newFiles.isEmpty())
						newFile = (File) newFiles.toArray()[0];
					ReceivedFilePath = newFile.getAbsolutePath();
					//success = true;
					//ForwardReceivedFilePath(ReceivedFilePath);
					if (ba.subExists(eventName + "_receivedfiles")) {
						ba.raiseEvent(B4JDragToMe.this, eventName + "_receivedfiles", new Object[] { fls });
					}
					event.setDropCompleted(success);
					event.consume();
				}
			}
		});

		control.setOnDragDropped(new EventHandler<DragEvent>() {

			@Override
			public void handle(DragEvent event) {
				ba.Log("OnDragDropped()");
				Dragboard dragboard = event.getDragboard();
				boolean success = false;
				File newFile = null;

				if (dragboard.hasUrl() && !dragboard.hasFiles()) {
					ReceivedFilePath = dragboard.getUrl();
					success = true;
					ForwardReceivedFilePath(ReceivedFilePath);
					event.setDropCompleted(success);
					event.consume();
				} else if (dragboard.hasFiles()) {
					List fls = new List();
					fls.Initialize();
					java.util.List<File> newFiles = dragboard.getFiles();
					ba.Log("NewFiles.Size = "+newFiles.size());
					for (File s : newFiles) {
						fls.Add(s.getAbsolutePath());
					}
					if (!newFiles.isEmpty())
						newFile = (File) newFiles.toArray()[0];
					ReceivedFilePath = newFile.getAbsolutePath();
					//success = true;
					//ForwardReceivedFilePath(ReceivedFilePath);
					if (ba.subExists(eventName + "_receivedfiles")) {
						ba.raiseEvent(B4JDragToMe.this, eventName + "_receivedfiles", new Object[] { fls });
					}
					event.setDropCompleted(success);
					//event.consume();

				}


			}
		});
	}

} // end of class
